from typing import List, Dict
from sequoia.settings.rl import make_continuous_task
from gym.envs.classic_control import CartPoleEnv
import numpy as np

# A Continuous task is a dict mapping from attributes to the values
# to be set on the environment:
ContinuousTask = Dict[str, float]


@make_continuous_task.register(CartPoleEnv)
def make_task_for_my_env(
    env: CartPoleEnv, step: int, change_steps: List[int], seed: int = None, **kwargs,
) -> ContinuousTask:
    # NOTE: task sampling should be reproducible given a `seed`.
    step_seed = seed * step if seed is not None else None
    rng = np.random.default_rng(step_seed)
    return {
        "gravity": 9.8 * rng.normal(1, 0.5),
        "masscart": 1.0 * rng.normal(1, 0.5),
        "masspole": 0.1 * rng.normal(1, 0.5),
        "length": 0.5 * rng.normal(1, 0.5),
        "force_mag": 10.0 * rng.normal(1, 0.5),
        "tau": 0.02 * rng.normal(1, 0.5),
    }