import gym
from sequoia.settings import Setting, Environment, Observations, Actions
from sequoia.methods import Method

class DemoModel:
    def forward(self, observations: Observations) -> Actions:
        ...

class DemoMethod(Method, target_setting=Setting):
    """ Pseudocode for a Method that targets a given Setting. """  
    def configure(self, setting: Setting):
        # Called by the setting before training begins.
        self.model = DemoModel(setting.observation_space, setting.action_space,
                               setting.reward_space, nb_tasks=setting.nb_tasks)
        self.optimizer = Adam(...)

    def fit(self, train_env: Environment, valid_env: Environment):
        # Train a model using these environments from the setting.
        # Note: all Environments are gym environments. More on this later.
        for epoch in range(self.n_epochs):
            self.model.train_epoch(train_env)
            self.model.validation_epoch(valid_env)

    def get_actions(self, observations: Observations) -> Actions:
        # Called by the setting for inference (at test-time).
        actions = self.model(observations)
        return actions

    def on_task_switch(self, task_id: Optional[int]):
        # Gets called on task boundaries, depending on the setting.
        self.model.prepare_for_new_task(new_task=task_id)